@@ -954,5 +954,30 @@ struct tone_zone builtin_zones[] =
 		.mfr1_level = -10,
 		.mfr2_level = -8,
 	},
+	{
+		.zone = 42,
+		.country = "pa",
+		.description = "Panama",
+		.ringcadence = { 2000, 4000 },
+		.tones = {
+			/* Reference: http://www.itu.int/ITU-T/inr/forms/files/tones-0203.pdf */
+			{ DAHDI_TONE_DIALTONE, "425" },
+			{ DAHDI_TONE_BUSY, "425/320,0/320" },
+			{ DAHDI_TONE_RINGTONE, "425/1200,0/4650" },
+			{ DAHDI_TONE_CONGESTION, "425/320,0/320" },
+			{ DAHDI_TONE_CALLWAIT, "425/180,0/180,425/180" },
+			/* RECALL DIAL TONE - not specified */
+			{ DAHDI_TONE_DIALRECALL, "!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,425" },
+			/* RECORD TONE - not specified */
+			{ DAHDI_TONE_RECORDTONE, "1400/500,0/15000" },
+			{ DAHDI_TONE_INFO, "!950/330,!1400/330,!1800/330,0" },
+			/* STUTTER TONE - not specified */
+			{ DAHDI_TONE_STUTTER, "!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,425" },
+		},
+		.dtmf_high_level = -9,
+		.dtmf_low_level = -11,
+		.mfr1_level = -7,
+		.mfr2_level = -8,
+	},
 	{ .zone = -1 }
 };
